#!/usr/bin/python

# Software to read data from Arduino cloudSensor, store the received data 
# and plot a temperature graph from the data
#
# by Panu Lahtinen, December 8, 2011
#
# Use as is, modify if need be, redistribute as free software.
#

import serial
import time
import datetime as dt
from collections import deque
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.dates as ds
import matplotlib.dates as mdates

# Function to request and read two comma separated values from Arduino.
# Returns temperature and IR temperature as floats
def get_data():
    # Request data
    arduino.write('r')
    time.sleep(0.1)
    # Read data
    result = arduino.readline()
    # Strip newline character away
    result = result.strip()
    # Split values to two variables and return as floating point numbers
    temp,ir_temp = result.split(sep,2)
    return (float(temp),float(ir_temp))

# Draw temperature graphs from the given data and store as PNG image
def draw_graph(t,ir,Y,m,d,M,H,S,imgfile):

    # Convert deque objects to numpy arrays
    t = np.array(t)
    ir = np.array(ir)

    # Form matplotlib dates
    times = np.zeros(len(Y))
    for i in range(0,len(Y)):
        times[i] = ds.date2num(dt.datetime(Y[i],m[i],d[i],H[i],M[i],S[i]))

    # X-axis (time) minimum and maximum values. Assume 24 hours of data 
    # for display
    xmax = times.max()
    # 24 hours of graph. Adjust if shorter/longer period is desired.
    # If longer period is wanted, adjust also the number of values 
    # stored in the buffers, adjust "n_vals" accordingly
    xmin = xmax - 24/24

    # Image size in pixels is converted to inches. Default DPI value is 100, 
    # thus the image size is easy to adjust
    width = 800/100.0
    height = 400/100.0
    
    # Create a figure and a subplot for the data
    fig = plt.figure(figsize=(width,height))
    ax = fig.add_subplot(111)

    # Plot data. Ambient in red, Sky in blue
    ax.plot(times,t,'r')
    ax.plot(times,ir,'b')

    # Format the tick marks for x-axis. Major tick marks for every hour, 
    # minor tick marks for every 15 minutes
    xFmt = mdates.DateFormatter('%H')
    hours = mdates.HourLocator()
    quarterHours = mdates.MinuteLocator(interval=15)
    ax.xaxis.set_major_locator(hours)
    ax.xaxis.set_major_formatter(xFmt)
    ax.xaxis.set_minor_locator(quarterHours)
    ax.set_xlim(xmin,xmax)
    ax.grid(True)
    
    # Add legend to upper left corner of the plot
    leg = plt.legend(('Ambient','Sky'), loc=2)
    # Set the legend to be semitransparent
    leg.get_frame().set_alpha(0.5)
    # Y-axis label
    plt.ylabel('Temperature')
    # X-axis label
    plt.xlabel('UTC Hour')
    # Title for the whole graph
    ax.set_title('Sky and ambient temperature in Perkkaa')
    # Add latest temperature values to the right side of the plot
    plt.figtext(0.91, 0.62, 'Latest:', color='k')
    plt.figtext(0.91, 0.55, str(t[-1])+' $^\circ$C',color='r')
    plt.figtext(0.91, 0.5, str(ir[-1])+' $^\circ$C',color='b')
    # Add update information to the bottom of the plot
    plt.figtext(0.65, 0.01, 'Updated: '+str(Y[-1])+'.'+str(m[-1]).zfill(2)+'.'+str(d[-1]).zfill(2)+' '+str(H[-1]).zfill(2)+'.'+str(M[-1]).zfill(2)+'.'+str(S[-1]).zfill(2), color='k')

    # Save figure to 
    plt.savefig(imgfile,format='png')


# Directory for the saved data
datadir = ''
# Image filename with full path
imgfile = 'latest.png'
# Arduino serial device address.
# In linux normally /dev/ttyUSB0, in windows COMxx or something?
dev = '/dev/ttyUSB0'
# Serial connection speed. Used exactly the same value that was set for 
# Arduino in cloudSensor.pde
speed = 38400
# Measure every 30 seconds
interval = 30
# Update image every two measurements (once a minute)
draw_every_Nth = 2
# Data separator for the incoming data
sep = ','

# Number of values to store in memory. One full day (24*60*60 seconds) is 
# assumed.
# If this is changed, you might also want to change "xmin" in draw_graph()
n_vals = int(24*60*60/interval)

# Prepare data buffers
temps = deque(range(0), n_vals) # Air temperature (Ambient)
ir_temps = deque(range(0), n_vals) # IR temperature (Sky)
Y = deque(range(0), n_vals) # Year
m = deque(range(0), n_vals) # Month
d = deque(range(0), n_vals) # Day
H = deque(range(0), n_vals) # Hour
M = deque(range(0), n_vals) # Minute
S = deque(range(0), n_vals) # Second

# Try to read the data from last two days to buffers
# If no earlier data available, start gathering
now = dt.datetime.utcnow()
one_day = dt.timedelta(days=1)
yesterday = now - one_day

datefile_yesterday = datadir+'/'+yesterday.strftime('%Y%m%d_cloud_sensor.txt')
datefile_today = datadir+'/'+now.strftime('%Y%m%d_cloud_sensor.txt')

# Read data for yesterday, if such exist
try:
    prev_data = np.loadtxt(datefile_yesterday, delimiter=',')
    for i in range(0,len(prev_data)):
        temps.append(prev_data[i,0].astype('float'))
        ir_temps.append(prev_data[i,1].astype('float'))
        Y.append(prev_data[i,2].astype('int'))
        m.append(prev_data[i,3].astype('int'))
        d.append(prev_data[i,4].astype('int'))
        H.append(prev_data[i,5].astype('int'))
        M.append(prev_data[i,6].astype('int'))
        S.append(prev_data[i,7].astype('int'))
except IOError:
    pass

# Read data for today, if such exist
try:
    prev_data = np.loadtxt(datefile_today, delimiter=',')
    for i in range(0,len(prev_data)):
        temps.append(prev_data[i,0].astype('float'))
        ir_temps.append(prev_data[i,1].astype('float'))
        Y.append(prev_data[i,2].astype('int'))
        m.append(prev_data[i,3].astype('int'))
        d.append(prev_data[i,4].astype('int'))
        H.append(prev_data[i,5].astype('int'))
        M.append(prev_data[i,6].astype('int'))
        S.append(prev_data[i,7].astype('int'))
except IOError:
    pass


# Open serial connection
arduino = serial.Serial(dev,speed)

n = 0
while True:
    req_time = time.time()
    # Request data from Arduino and read it from serial interface
    temp,ir_temp = get_data()
    n += 1

    # Use UTC time. Swap these if local time is preferred. Note also that 
    # UTC is mentioned in the graph!
    stamp = dt.datetime.utcnow()
    # stamp = dt.datetime.now()

    # Updated data buffers with latest values. If buffer is full, oldest 
    # data will be automatically removed.
    temps.append(temp)
    ir_temps.append(ir_temp)
    Y.append(int(stamp.strftime("%Y"))) # Year
    m.append(int(stamp.strftime("%m"))) # Month
    d.append(int(stamp.strftime("%d"))) # Day
    H.append(int(stamp.strftime("%H"))) # Hour
    M.append(int(stamp.strftime("%M"))) # Minute
    S.append(int(stamp.strftime("%S"))) # Second

    # Save data to daily file
    datefile = datadir+'/'+stamp.strftime('%Y%m%d_cloud_sensor.txt')
    f = open(datefile,'a')

    # Format data to CSV
    data = str(temp)+','+str(ir_temp)+","+stamp.strftime("%Y,%m,%d,%H,%M,%S")
    print data
    f.write(data+'\n')
    f.close()
    
    # If we have collected n samples, draw the graph
    if n >= draw_every_Nth:
        draw_graph(temps,ir_temps,Y,m,d,M,H,S,imgfile)
        n = 0

    ready_time = time.time()

    # Wait for the next round
    time.sleep(interval+req_time-ready_time)
    

